# !/usr/bin/env Python3
# -*- coding:utf-8 -*-
# Team : HiLab
# Author：Yu Tao
# Date ：2022/6/28 19:53
# Tool ：PyCharm
# Email : yuao_11260103@163.com
# Project : msDAWG
# ------ ------
# reproduce SHAP to feature select
# ------ ------

from load_data import *
from datetime import datetime
import sys
from scripts.feature_select import shap_select
from scripts.evaluation import all_evaluate

dt = datetime.today()
file_name = str(dt.day) + '-' + str(dt.hour) + '-' + str(dt.minute)

log_print = open(file_name + '.log', 'w')
sys.stdout = log_print
sys.stderr = log_print
datasets = ['HIP_CER', 'ICC_rms', 'SIMS', 'OvarianL91-162',
            'PanINDataH101-80',
            'OvarianL16-100-100',
            'JNCIL69-190-63',
            'OvarianH95-121']

for i in range(8):
    print(datasets[i])
    X_train, y_train = get_train_data(i)
    X_test, y_test = get_test_data(i)
    X = np.load('data/' + datasets[i] + '_X.npy')
    data_df = pd.read_pickle('data/' + datasets[i] + '.pkl')
    mz = data_df.columns.values[:-1]
    mz = mz.astype(float)
    data_dict = {'X_train': X_train,
                 'y_train': y_train,
                 'X_test': X_test,
                 'y_test': y_test}
    shap_ranked_index = shap_select(data_dict, X)

    precision_all = []
    recall_all = []
    acc_all = []
    auc_all = []
    feature_num = []
    for j in range(1, 101):
        print(j)
        precision, recall, accuracy, aucc = all_evaluate(X_train[:, shap_ranked_index[:j]],
                                                         X_test[:, shap_ranked_index[:j]],
                                                         y_train,
                                                         y_test)
        feature_num.append(j)
        precision_all.append(precision)
        recall_all.append(recall)
        acc_all.append(accuracy)
        auc_all.append(aucc)
    best_feature_index = auc_all.index(max(auc_all))
    best_feature_num = feature_num[best_feature_index]
    best_feature_auc = auc_all[best_feature_index]
    selected_feature_index = shap_ranked_index[:best_feature_num]
    print('best_feature_num:', best_feature_num)
    print('best_feature_auc:', best_feature_auc)
    print('selected_feature_index:', selected_feature_index)
    precision_all = np.array(precision_all)
    recall_all = np.array(recall_all)
    acc_all = np.array(acc_all)
    auc_all = np.array(auc_all)
    feature_num = np.array(feature_num)

    precision_all = precision_all[None]
    recall_all = recall_all[None]
    acc_all = acc_all[None]
    auc_all = auc_all[None]
    feature_num = feature_num[None]
    score = np.concatenate([feature_num, precision_all], axis=0)
    score = np.concatenate([score, recall_all], axis=0)
    score = np.concatenate([score, acc_all], axis=0)
    score = np.concatenate([score, auc_all], axis=0)
    print(mz[shap_ranked_index])
    np.savetxt('mz/' + datasets[i] + '_mz.csv', mz[shap_ranked_index], delimiter=',', fmt='%.04f')
    np.savetxt('scores/' + datasets[i] + '_baseline.csv', score, delimiter=',', fmt='%.06f')
