# !/usr/bin/env Python3
# -*- coding:utf-8 -*-
# Team : HiLab
# Author：Yu Tao
# Date ：2022/6/20 20:17
# Tool ：PyCharm
# Email : yuao_11260103@163.com
# Project : msDAWG
import numpy as np
import pandas as pd
from sklearn.model_selection import train_test_split

datasets = ['HIP_CER', 'ICC_rms', 'SIMS', 'OvarianL91-162',
            'PanINDataH101-80',
            'OvarianL16-100-100',
            'JNCIL69-190-63']


def load_all_data(data_index):
    """
    load all dataset

    :param data_index: int
    :return: data_x, data_y, data_label, dataset -> arr, arr, ls, str
    """
    dataset = datasets[data_index]  # change dataset
    data_df = pd.read_pickle('data/' + dataset + '.pkl')
    data_label = list(set(data_df['type']))
    data_label.sort()
    data_x = np.load('data/' + dataset + '_X.npy')
    data_y = np.load('data/' + dataset + '_y.npy', allow_pickle=True).astype(np.float32)
    return data_x, data_y, data_label, dataset


def split_train_test(data_x, data_y):
    """
    8-2,shuffle,stratify,random_state=19

    :param data_x:arr
    :param data_y:arr
    :return:X_train, X_test, y_train, y_test -> arr, arr, arr, arr
    """
    X_train, X_test, y_train, y_test = train_test_split(data_x,
                                                        data_y,
                                                        test_size=0.2,
                                                        random_state=19,
                                                        shuffle=True,
                                                        stratify=data_y)
    return X_train, X_test, y_train, y_test


def get_train_data(data_index):
    """
    get training data

    :rtype: object
    :param data_index
    :return:X_train, y_train -> arr, arr
    """
    X_train = np.load('data/82split/' + datasets[data_index] + '_train_X.npy')
    y_train = np.load('data/82split/' + datasets[data_index] + '_train_Y.npy', allow_pickle=True).astype(np.float32)
    return X_train, y_train


def get_test_data(data_index):
    """
    get testing data

    :param data_index:
    :return:X_test, y_test -> arr, arr
    """
    X_test = np.load('data/82split/' + datasets[data_index] + '_test_X.npy')
    y_test = np.load('data/82split/' + datasets[data_index] + '_test_Y.npy', allow_pickle=True).astype(np.float32)
    return X_test, y_test


def get_change_test_data(data_index):
    """
    get test data

    :param data_index
    :return:X_test -> arr, arr
    """
    X_test = np.load('data/Sps/' + datasets[data_index] + '_test_X.npy')
    return X_test

