# !/usr/bin/env Python3
# -*- coding:utf-8 -*-
# Team:HiLab
# Author:Yu Tao
# Date:2022/6/30 18:57
# Tool:PyCharm
# Email:yuao_11260103@163.com
# Project:msDAWG
import torch
import random
import numpy as np
from scripts.model import setup_seed, three_coder_AE
import torch.nn.functional as F
from torch.utils.data import DataLoader
from param_class import Dataset_Param
from load_data import split_train_test
from torch import nn, optim
import matplotlib.pyplot as plt
from matplotlib.pyplot import subplots
from datetime import datetime

dt = datetime.today()
file_name = str(dt.day) + '-' + str(dt.hour) + '-' + str(dt.minute)

device = torch.device("cuda:0" if torch.cuda.is_available() else "cpu")
# Assuming that we are on a CUDA machine, this should print a CUDA device:
print(device)
# net.to(device)

seed = 19
setup_seed(seed)
torch.manual_seed(seed)
torch.cuda.manual_seed_all(seed)
np.random.seed(seed)
random.seed(seed)
torch.backends.cudnn.deterministic = True


class MyDataSet(torch.utils.data.Dataset):
    def __init__(self, data, label):
        self.data = torch.from_numpy(data).float()
        self.label = torch.from_numpy(label).float()  # torch.from_numpy(label)
        self.length = data.shape[0]

    def __getitem__(self, index):
        return self.data[index], self.label[index]

    def __len__(self):
        return self.length


def train_module(dataloader, model, optimizer):
    num_batches = len(dataloader)
    train_loss = 0
    for i, data in enumerate(dataloader, 0):
        input, label = data
        input, label = input.to(device), label.to(device)
        _, _, _, _, _, _, output, _, _, out_label = model(input)

        loss_mse = F.mse_loss(output, input)
        loss_ce = F.cross_entropy(out_label, label.long())
        loss = (loss_mse + loss_ce) / 2
        train_loss += loss.item()

        optimizer.zero_grad()
        loss.backward()
        optimizer.step()

    return train_loss / num_batches


def test_module(dataloader, model):
    num_batches = len(dataloader)
    test_loss = 0
    with torch.no_grad():
        for i, data in enumerate(dataloader, 0):
            input, label = data
            input, label = input.to(device), label.to(device)

            _, _, _, _, _, _, output, _, _, out_label = model(input)
            loss_mse = F.mse_loss(output, input)
            loss_ce = F.cross_entropy(out_label, label.long())

            loss = (loss_mse + loss_ce) / 2
            test_loss += loss.item()
    return test_loss / num_batches


def three_coder_AE_feature_construct(X, y, Dataset_Param):
    print('---start three_coder_AE_feature_construct---')

    # ========param===========
    data_name = Dataset_Param.name
    param_batch_size = Dataset_Param.batch_size
    param_lr = Dataset_Param.lr
    param_epoch = Dataset_Param.epoch
    # ========================

    X_train, X_test, y_train, y_test = split_train_test(X, y)
    data_dict = {'X_train': X_train, 'X_test': X_test, 'y_train': y_train, 'y_test': y_test}

    print("data loading...")
    train_data_set = MyDataSet(data_dict['X_train'], data_dict['y_train'])
    train_dataloader = DataLoader(train_data_set, batch_size=param_batch_size, shuffle=True, drop_last=True)
    test_data_set = MyDataSet(data_dict['X_test'], data_dict['y_test'])
    test_dataloader = DataLoader(test_data_set, batch_size=param_batch_size, shuffle=True, drop_last=True)

    print('model constructing....')
    net = three_coder_AE(data_dict['X_train'].shape[1])
    net.to(device)
    optimizer = optim.Adam(net.parameters(), lr=param_lr)
    ls = []
    lss = []
    for i in range(param_epoch):  # epoch
        print(i)
        net.train()
        ls.append(train_module(train_dataloader, net, optimizer))
        net.eval()
        lss.append(test_module(test_dataloader, net))
    print("OK")

    # ---观察损失曲线---
    print('Start drawing....')
    fig, axes = subplots(1, 1)
    axes.plot(ls, label='train-loss')
    axes.plot(lss, label='test-loss')
    axes.set_xlabel('epoch', fontsize=13)
    plt.title(data_name + '_' + str(param_batch_size) + '_' + str(param_lr) + '-trainset-3coder_AE_feature_construct')
    plt.legend(fontsize=12)
    plt.savefig(
        'fig/' + data_name + '_' + str(param_batch_size) + '_' + str(param_lr) + '_' + str(param_epoch) + '.png')
    # torch.save(net, 'model/' + data_name + '_' + str(param_batch_size) + '_' + str(param_lr) + '_' + str(param_epoch) + '.pkl')
    return net
