# !/usr/bin/env Python3
# -*- coding:utf-8 -*-
# Team:HiLab
# Author:Yu Tao
# Date:2022/7/9 17:46
# Tool:PyCharm
# Email:yuao_11260103@163.com
# Project:msDAWG
from scripts.feature_select import *
from scripts.evaluation import *
from param_class import Dataset_Param
import torch
import random
import warnings
import sys
from datetime import datetime
from wGAN_attention_dual_AE_FC import wgan_attention_dualAE_FC
from sklearn.ensemble import GradientBoostingClassifier
from load_data import *

warnings.filterwarnings('ignore')
seed = 42
torch.manual_seed(seed)
torch.cuda.manual_seed_all(seed)
np.random.seed(seed)
random.seed(seed)
torch.backends.cudnn.deterministic = True

dt = datetime.today()
file_name = str(dt.day) + '-' + str(dt.hour) + '-' + str(dt.minute)

# log_print = open(file_name + '.log', 'w')
# sys.stdout = log_print
# sys.stderr = log_print

HIP = Dataset_Param(0, 20, 1e-5, 200)
ICC12 = Dataset_Param(1, 10, 1e-5, 200)
SIMS = Dataset_Param(2, 10, 1e-4, 100)
OvarianL162 = Dataset_Param(3, 10, 1e-4, 100)
PanIN = Dataset_Param(4, 20, 1e-4, 100)
OvarianL100 = Dataset_Param(5, 20, 5e-4, 200)
JNCIL = Dataset_Param(6, 20, 5e-5, 200)
datasets = [HIP, ICC12, SIMS, OvarianL162, PanIN, OvarianL100, JNCIL]
start = False
ok = True
print("start wGAN+dualAE+attention GB sandwitch")
for data in datasets:
    print("start:", data.name)
    X_train = np.load('data/GB_FS/' + data.name + '_GB_train_X.npy')
    X_test = np.load('data/GB_FS/' + data.name + '_GB_test_X.npy')
    y_train = np.load('data/82split/' + data.name + '_train_Y.npy', allow_pickle=True).astype(np.float32)
    y_test = np.load('data/82split/' + data.name + '_test_Y.npy', allow_pickle=True).astype(np.float32)
    if start:
        model = wgan_attention_dualAE_FC(X_train, y_train, Dataset_Param=data)
        torch.save(model,
                   'model/wGAN_attention_dualAE_model/' + data.name + '_' + str(data.batch_size) + '_' + str(
                       data.lr) + '_' + str(
                       data.epoch) + '.pkl')

    if ok:
        model = torch.load(
            'model/wGAN_attention_dualAE_model/' + data.name + '_' + str(data.batch_size) + '_' + str(
                data.lr) + '_' + str(
                data.epoch) + '.pkl')
        # construct
        tuple_out_train = model(torch.tensor(X_train).to(torch.float32))
        tuple_out_test = model(torch.tensor(X_test).to(torch.float32))
        # concat all layers
        concate_feature_train = X_train
        concate_feature_test = X_test
        for i in range(1, len(tuple_out_train)):
            out_train = tuple_out_train[i].detach().numpy()
            out_test = tuple_out_test[i].detach().numpy()
            concate_feature_train = np.concatenate((concate_feature_train, out_train), axis=1)
            concate_feature_test = np.concatenate((concate_feature_test, out_test), axis=1)

        data_dict = {'X_train': concate_feature_train, 'X_test': concate_feature_test, 'y_train': y_train,
                     'y_test': y_test}
        X = np.concatenate((concate_feature_train, concate_feature_test), axis=0)
        y = np.concatenate((y_train, y_test))

        precision_all = []
        recall_all = []
        acc_all = []
        auc_all = []
        feature_num = []
        for j in range(1, 101):
            print(j)
            data_dict = {'X_train': X,
                         'y_train': y,
                         'X_test': X,
                         'y_test': y}
            model = GradientBoostingClassifier(random_state=1)
            data_X_selected = SFM(model, data_dict=data_dict, feature_num=j)
            X_train, X_test, y_train, y_test = split_train_test(data_X_selected, y)
            precision, recall, accuracy, aucc = all_evaluate(X_train,
                                                             X_test,
                                                             y_train,
                                                             y_test)
            feature_num.append(j)
            precision_all.append(precision)
            recall_all.append(recall)
            acc_all.append(accuracy)
            auc_all.append(aucc)
        best_feature_index = auc_all.index(max(auc_all))
        best_feature_num = feature_num[best_feature_index]
        best_feature_precision = precision_all[best_feature_index]
        best_feature_recall = recall_all[best_feature_index]
        best_feature_acc = acc_all[best_feature_index]
        best_feature_auc = auc_all[best_feature_index]
        print('best_feature_num:', best_feature_num)
        print('best_feature_precision:', best_feature_precision)
        print('best_feature_recall:', best_feature_recall)
        print('best_feature_acc:', best_feature_acc)
        print('best_feature_auc:', best_feature_auc)
        # precision_all = np.array(precision_all)
        # recall_all = np.array(recall_all)
        # acc_all = np.array(acc_all)
        # auc_all = np.array(auc_all)
        # feature_num = np.array(feature_num)
        #
        # precision_all = precision_all[None]
        # recall_all = recall_all[None]
        # acc_all = acc_all[None]
        # auc_all = auc_all[None]
        # feature_num = feature_num[None]
        # score = np.concatenate([feature_num, precision_all], axis=0)
        # score = np.concatenate([score, recall_all], axis=0)
        # score = np.concatenate([score, acc_all], axis=0)
        # score = np.concatenate([score, auc_all], axis=0)
        # np.savetxt('scores/' + data.name + '_GAN_attention_dualAE_GB_scores' + file_name + '.csv', score, delimiter=',')
